"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const filterTypes_1 = require("./filterTypes");
const utils_1 = require("./utils");
const Filter_1 = require("./Filter");
const FilterGroup_1 = __importDefault(require("./FilterGroup"));
const filterConditions_1 = require("./filterConditions");
require("../../../../../css/filter-card.css");
require("../../../../../css/margins.css");
const ADD_FILTER_SELECT_TITLE = '+ Add Filter';
/*
    Component that contains all that one needs to filter!
*/
class FilterCard extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.state = {
            filters: props.filters,
            operator: props.operator,
            stepID: '',
            filterSelect: ADD_FILTER_SELECT_TITLE
        };
        this.sendFilterUpdateMessage = this.sendFilterUpdateMessage.bind(this);
        this.addFilter = this.addFilter.bind(this);
        this.addFilterGroup = this.addFilterGroup.bind(this);
        this.addFilterToGroup = this.addFilterToGroup.bind(this);
        this.deleteFilter = this.deleteFilter.bind(this);
        this.deleteFilterFromGroup = this.deleteFilterFromGroup.bind(this);
        this.setFilter = this.setFilter.bind(this);
        this.setFilterInGroup = this.setFilterInGroup.bind(this);
        this.setOperator = this.setOperator.bind(this);
        this.setOperatorInGroup = this.setOperatorInGroup.bind(this);
    }
    /*
        Before sending the displayed filters, we parse all the number filters from strings
        to numbers, and then filter out all of the invalid filters (as to not cause errors)
    */
    sendFilterUpdateMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            /*
                A filter is invalid if:
                1. It should have an input, and it does not
                2. It is a number filter with a string input, or with a non-valid number input
            */
            const isValidFilter = (filter) => {
                if (filterConditions_1.CONDITIONS_WITH_NO_INPUT.includes(filter.condition)) {
                    return true;
                }
                if (filter.type === 'number') {
                    return typeof filter.value !== 'string' && !isNaN(filter.value);
                }
                if (filter.type === 'string' || filter.type === 'datetime') {
                    return filter.value !== '';
                }
                return true;
            };
            /*
                The frontend stores number filters as strings, and so we parse them to
                numbers before sending them to the backend
            */
            const parseFilter = (filter) => {
                if (filter.type === 'number' && typeof filter.value === 'string') {
                    return {
                        type: filter.type,
                        condition: filter.condition,
                        value: parseFloat(filter.value)
                    };
                }
                return filter;
            };
            // To handle decimals, we allow decimals to be submitted, and then just
            // parse them before they are sent to the back-end
            const parsedFilters = this.state.filters.map((filterOrGroup) => {
                if (filterTypes_1.isFilterGroup(filterOrGroup)) {
                    return {
                        filters: filterOrGroup.filters.map((filter) => {
                            return parseFilter(filter);
                        }),
                        operator: filterOrGroup.operator
                    };
                }
                else {
                    return parseFilter(filterOrGroup);
                }
            });
            const filtersToApply = parsedFilters.map((filterOrGroup) => {
                // Filter out these incomplete filters from the group
                if (filterTypes_1.isFilterGroup(filterOrGroup)) {
                    return {
                        filters: filterOrGroup.filters.filter((filter) => {
                            return isValidFilter(filter);
                        }),
                        operator: filterOrGroup.operator
                    };
                }
                else {
                    return filterOrGroup;
                }
            }).filter((filterOrGroup) => {
                // Filter out the groups if they have no valid filters in them
                if (filterTypes_1.isFilterGroup(filterOrGroup)) {
                    return filterOrGroup.filters.length > 0;
                }
                // And then we filter the non group filters to be non-empty
                return isValidFilter(filterOrGroup);
            });
            const stepID = yield this.props.mitoAPI.sendFilterMessage(this.props.selectedSheetIndex, this.props.columnHeader, filtersToApply, this.state.operator, this.state.stepID);
            this.setState({ stepID: stepID });
        });
    }
    /*
        Adds a new, blank filter to the end of the filters list
    */
    addFilter() {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.push(utils_1.getEmptyFilterData(this.props.columnType));
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Creates a new filter group (at the bottom) with a single empty
        filter
    */
    addFilterGroup() {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.push({
                filters: [
                    utils_1.getEmptyFilterData(this.props.columnType)
                ],
                operator: 'And'
            });
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Adds a blank new filter to the end of a specific group
    */
    addFilterToGroup(groupIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (filterTypes_1.isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we add a new filter to it
                filterGroup.filters.push(utils_1.getEmptyFilterData(this.props.columnType));
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Deletes a filter that is at the given index in the main
        filter list.
    */
    deleteFilter(filterIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.splice(filterIndex, 1);
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Deletes a filter that is at a given index in filter list
        of a specific filter group
    */
    deleteFilterFromGroup(groupIndex, filterIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (filterTypes_1.isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we delete the filter
                // at the passed filterIndex
                filterGroup.filters.splice(filterIndex, 1);
                // If there are no filters left in this group, then we remove the entire group
                if (filterGroup.filters.length === 0) {
                    newFilters.splice(groupIndex, 1);
                }
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets a filter at the given index to the new filter value
    */
    setFilter(filterIndex, filter) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters[filterIndex] = filter;
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets a filter at the given filterIndex in the specific group at the given
        groupIndex to the new filter value
    */
    setFilterInGroup(groupIndex, filterIndex, filter) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (filterTypes_1.isFilterGroup(filterGroup)) {
                filterGroup.filters[filterIndex] = filter;
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets the operator that combines the highest level filters and
        filter groups
    */
    setOperator(operator) {
        this.setState({ operator: operator }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets the operator that combines a specific filter group
    */
    setOperatorInGroup(groupIndex, operator) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (filterTypes_1.isFilterGroup(filterGroup)) {
                filterGroup.operator = operator;
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    render() {
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("div", { className: 'filter-modal-section-title mt-25px' },
                react_1.default.createElement("p", null, " Filter ")),
            react_1.default.createElement("div", { className: "filter-modal-centering-container" }, this.state.filters.map((filterOrGroup, index) => {
                if (filterTypes_1.isFilterGroup(filterOrGroup)) {
                    return (
                    /*
                        If the FilterGroup is the first Filter or FilterGroup
                        in the FilterCard, add a 'Where'
                    */
                    react_1.default.createElement("div", { className: 'filter-group-and-operator-div' },
                        index === 0 &&
                            react_1.default.createElement("div", { className: 'filter-where-label filter-object-item mr-10px' }, "Where"),
                        index !== 0 &&
                            react_1.default.createElement("div", { className: 'filter-select filter-operator mr-10px' },
                                react_1.default.createElement("select", { className: 'filter-select filter-operator mr-10px', value: this.state.operator, onChange: (e) => { this.setOperator(e.target.value); } },
                                    react_1.default.createElement("option", { value: 'And' }, "And"),
                                    react_1.default.createElement("option", { value: 'Or' }, "Or"))),
                        react_1.default.createElement(FilterGroup_1.default, { key: index, mainOperator: this.state.operator, filters: filterOrGroup.filters, groupOperator: filterOrGroup.operator, setFilter: (filterIndex, newFilter) => {
                                this.setFilterInGroup(index, filterIndex, newFilter);
                            }, setOperator: (newOperator) => {
                                this.setOperatorInGroup(index, newOperator);
                            }, deleteFilter: (filterIndex) => {
                                this.deleteFilterFromGroup(index, filterIndex);
                            }, addFilter: () => this.addFilterToGroup(index) })));
                }
                else {
                    return (react_1.default.createElement(Filter_1.Filter, { first: index === 0, key: index, filter: filterOrGroup, operator: this.state.operator, displayOperator: true, setFilter: (newFilter) => {
                            this.setFilter(index, newFilter);
                        }, setOperator: this.setOperator, deleteFilter: () => { this.deleteFilter(index); } }));
                }
            })),
            react_1.default.createElement("div", { className: 'filter-card-buttons-container' },
                react_1.default.createElement("select", { className: 'filter-add-filter-button', value: ADD_FILTER_SELECT_TITLE, onChange: (e) => {
                        if (e.target.value === 'filter') {
                            this.addFilter();
                        }
                        else {
                            this.addFilterGroup();
                        }
                        // Set this so it's always what is selected
                        this.setState({ filterSelect: ADD_FILTER_SELECT_TITLE });
                    } },
                    react_1.default.createElement("option", { selected: true, hidden: true }, ADD_FILTER_SELECT_TITLE),
                    react_1.default.createElement("option", { value: 'filter' }, "+ Add a Filter"),
                    react_1.default.createElement("option", { value: 'filter-group' }, "+ Add a group of filters")))));
    }
}
exports.default = FilterCard;
//# sourceMappingURL=FilterCard.js.map